@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2009 NLNet Labs. All rights reserved.
+ * Copyright (c) 2009-2018 NLNet Labs.
+ * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -21,7 +22,6 @@
  * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
  * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
  * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
  */
 
 /**
@@ -48,6 +48,7 @@
 #include "log.h"
 #include "locks.h"
 #include "util.h"
+#include "utilities.h"
 
 static const char* schedule_str = "scheduler";
 
@@ -189,10 +190,6 @@ schedule_create()
 {
     schedule_type* schedule;
     CHECKALLOC(schedule = (schedule_type*) malloc(sizeof(schedule_type)));
-    if (!schedule) {
-        ods_log_error("[%s] unable to create: malloc failed", schedule_str);
-        return NULL;
-    }
 
     schedule->tasks = ldns_rbtree_create(task_compare_time_then_ttuple);
     schedule->tasks_by_name = ldns_rbtree_create(task_compare_ttuple);
