@@ -38,13 +38,12 @@
 #include "log.h"
 #include "str.h"
 #include "clientpipe.h"
+#include "longgetopt.h"
 
 #include "enforcer/enforce_cmd.h"
 
 static const char *module_str = "enforce_cmd";
 
-#define MAX_ARGS 4
-
 /**
  * Print help for the 'enforce' command
  *
@@ -74,16 +73,12 @@ help(int sockfd)
  *
  */
 static int
-run(int sockfd, cmdhandler_ctx_type* context, const char *cmd)
+run(cmdhandler_ctx_type* context, int argc, char* argv[])
 {
-	time_t t_next;
-	task_type *task;
-	char *buf;
-	int argc = 0;
-	char const *argv[MAX_ARGS];
+    int sockfd = context->sockfd;
+    struct longgetopt optctx;
 	int long_index = 0, opt = 0;
 	char const *zone_name = NULL;
-	int pos;
         db_connection_t* dbconn = getconnectioncontext(context);
         engine_type* engine = getglobalcontext(context);
 
@@ -92,33 +87,15 @@ run(int sockfd, cmdhandler_ctx_type* context, const char *cmd)
 		{0, 0, 0, 0}
 	};
 
-	ods_log_debug("[%s] %s command", module_str, enforce_funcblock.cmdname);
-
-	if (!cmd) return -1;
-
-	if (!(buf = strdup(cmd))) {
-		client_printf_err(sockfd, "memory error\n");
-		return -1;
-	}
-	argc = ods_str_explode(buf, MAX_ARGS, argv);
-	if (argc == -1) {
-		client_printf_err(sockfd, "too many arguments\n");
-		ods_log_error("[%s] too many arguments for %s command",
-				module_str, enforce_funcblock.cmdname);
-		free(buf);
-		return -1;
-	}
-
-	optind = 0;
-	while ((opt = getopt_long(argc, (char* const*)argv, "z:", long_options, &long_index)) != -1) {
+	for(opt = longgetopt(argc, argv, "z:", long_options, &long_index, &optctx); opt != -1;
+	    opt = longgetopt(argc, argv, NULL, long_options, &long_index, &optctx)) {
 		switch (opt) {
 			case 'z':
-				zone_name = optarg;
+				zone_name = optctx.optarg;
 				break;
 			default:
 				client_printf_err(sockfd, "unknown arguments\n");
-				ods_log_error("[%s] unknown arguments for %s command",
-						module_str, enforce_funcblock.cmdname);
+				ods_log_error("[%s] unknown arguments for enforce command", module_str);
 				return -1;
 
 		}
@@ -129,10 +106,9 @@ run(int sockfd, cmdhandler_ctx_type* context, const char *cmd)
 	} else {
 		enforce_task_flush_all(engine, dbconn);
 	}
-	free(buf);
 	return 0;
 }
 
 struct cmd_func_block enforce_funcblock = {
-	"enforce", &usage, &help, NULL, &run
+	"enforce", &usage, &help, NULL, NULL, &run, NULL
 };
