@@ -47,7 +47,7 @@
  * @param[in] keyfactory
  * @return time_t Time the function wishes to be called again.
  * */
-time_t
+extern time_t
 update(engine_type *engine, db_connection_t *dbconn, zone_db_t *zone, policy_t const *policy, time_t now, int *zone_updated);
 
 #endif /* _ENFORCER_ENFORCER_H_ */
