@@ -36,6 +36,7 @@
 #include "log.h"
 #include "str.h"
 #include "clientpipe.h"
+#include "longgetopt.h"
 #include "db/key_data.h"
 #include "keystate/keystate_ds.h"
 
@@ -69,12 +70,12 @@ help(int sockfd)
 }
 
 static int
-run(int sockfd, cmdhandler_ctx_type* context, const char *cmd)
+run(cmdhandler_ctx_type* context, int argc, char* argv[])
 {
 	int error;
         db_connection_t* dbconn = getconnectioncontext(context);
         engine_type* engine = getglobalcontext(context);
-	error = run_ds_cmd(sockfd, cmd, dbconn,
+	error = run_ds_cmd(context, argc, argv, dbconn,
 		KEY_DATA_DS_AT_PARENT_RETRACTED,
 		KEY_DATA_DS_AT_PARENT_UNSUBMITTED, engine);
 	if (error == 0) {
@@ -85,5 +86,5 @@ run(int sockfd, cmdhandler_ctx_type* context, const char *cmd)
 }
 
 struct cmd_func_block key_ds_gone_funcblock = {
-	"key ds-gone", &usage, &help, NULL, &run
+	"key ds-gone", &usage, &help, NULL, NULL, &run, NULL
 };
