@@ -38,6 +38,7 @@
 #include "log.h"
 #include "str.h"
 #include "clientpipe.h"
+#include "longgetopt.h"
 
 #include "db/key_state.h"
 #include "db/hsm_key.h"
@@ -356,13 +357,12 @@ printdebugparsablekey(int sockfd, zone_db_t* zone, key_data_t* key, char* tchang
 }
 
 static int
-run(int sockfd, cmdhandler_ctx_type* context, const char *cmd)
+run(cmdhandler_ctx_type* context, int argc, char* argv[])
 {
-    char buf[ODS_SE_MAXLINE];
-    #define NARGV 12
-    const char *argv[NARGV];
-    int success, argIndex;
-    int argc = 0, bVerbose = 0, bDebug = 0, bFull = 0, bParsable = 0, bAll = 0;
+    int sockfd = context->sockfd;
+    struct longgetopt optctx;
+    int success;
+    int bVerbose = 0, bDebug = 0, bFull = 0, bParsable = 0, bAll = 0;
     int long_index = 0, opt = 0;
     const char* keytype = NULL;
     const char* keystate = NULL;
@@ -381,22 +381,8 @@ run(int sockfd, cmdhandler_ctx_type* context, const char *cmd)
         {0, 0, 0, 0}
     };
 
-    ods_log_debug("[%s] %s command", module_str, key_list_funcblock.cmdname);
-
-    /* Use buf as an intermediate buffer for the command. */
-    strncpy(buf, cmd, sizeof (buf));
-    buf[sizeof (buf) - 1] = '\0';
-
-    /* separate the arguments */
-    argc = ods_str_explode(buf, NARGV, argv);
-    if (argc == -1) {
-        ods_log_error("[%s] too many arguments for %s command",
-                module_str, key_list_funcblock.cmdname);
-        client_printf_err(sockfd, "too many arguments\n");
-        return -1;
-    }
-    optind = 0;
-    while ((opt = getopt_long(argc, (char* const*)argv, "vdfpz:t:e:a", long_options, &long_index) ) != -1) {
+    for(opt = longgetopt(argc, argv, "vdfpz:t:e:a", long_options, &long_index, &optctx); opt != -1;
+        opt = longgetopt(argc, argv, NULL,          long_options, &long_index, &optctx)) {
         switch (opt) {
             case 'v':
                 bVerbose = 1;
@@ -411,21 +397,20 @@ run(int sockfd, cmdhandler_ctx_type* context, const char *cmd)
                 bParsable = 1;
                 break;
             case 'z':
-                zonename = optarg;
+                zonename = optctx.optarg;
                 break;
             case 't':
-                keytype = optarg;
+                keytype = optctx.optarg;
                 break;
             case 'e':
-                keystate = optarg;
+                keystate = optctx.optarg;
                 break;
             case 'a':
                 bAll = 1;
                 break;
             default:
                 client_printf_err(sockfd, "unknown arguments\n");
-                ods_log_error("[%s] unknown arguments for %s command",
-                              module_str, key_list_funcblock.cmdname);
+                ods_log_error("[%s] unknown arguments for key list command", module_str);
                 return -1;
         }
     }
@@ -454,10 +439,9 @@ run(int sockfd, cmdhandler_ctx_type* context, const char *cmd)
             client_printf_err(sockfd, "-p option only available in combination with -v and -d.\n");
         success = perform_keystate_list(sockfd, dbconn, zonename, keytype, keystate, &printcompatheader, &printcompatkey);
     }
-
     return success;
 }
 
 struct cmd_func_block key_list_funcblock = {
-	"key list", &usage, &help, NULL, &run
+	"key list", &usage, &help, NULL, NULL, &run, NULL
 };
