@@ -38,6 +38,7 @@
 #include "log.h"
 #include "str.h"
 #include "clientpipe.h"
+#include "longgetopt.h"
 
 #include "keystate/rollover_list_cmd.h"
 
@@ -167,12 +168,11 @@ help(int sockfd)
 }
 
 static int
-run(int sockfd, cmdhandler_ctx_type* context, const char *cmd)
+run(cmdhandler_ctx_type* context, int argc, char* argv[])
 {
-	#define NARGV 4
-	char buf[ODS_SE_MAXLINE];
-	const char *argv[NARGV];
-	int argc = 0, long_index = 0, opt = 0;
+    int sockfd = context->sockfd;
+    struct longgetopt optctx;
+	int long_index = 0, opt = 0;
 	const char *zone = NULL;
         db_connection_t* dbconn = getconnectioncontext(context);
 
@@ -181,31 +181,15 @@ run(int sockfd, cmdhandler_ctx_type* context, const char *cmd)
 		{0, 0, 0, 0}
 	};
 	
-	ods_log_debug("[%s] %s command", module_str, rollover_list_funcblock.cmdname);
-	
-	/* Use buf as an intermediate buffer for the command.*/
-	strncpy(buf, cmd,sizeof(buf));
-	buf[sizeof(buf)-1] = '\0';
-	
-	/* separate the arguments*/
-	argc = ods_str_explode(buf, NARGV, argv);
-	if (argc == -1) {
-		client_printf_err(sockfd, "too many arguments\n");
-		ods_log_error("[%s] too many arguments for %s command",
-				module_str, rollover_list_funcblock.cmdname);
-		return -1;
-	}
-
-	optind = 0;
-	while ((opt = getopt_long(argc, (char* const*)argv, "z:", long_options, &long_index)) != -1) {
+	for(opt = longgetopt(argc, argv, "z:", long_options, &long_index, &optctx); opt != -1;
+	    opt = longgetopt(argc, argv, NULL, long_options, &long_index, &optctx)) {
 		switch (opt) {
 			case 'z':
-				zone = optarg;
+				zone = optctx.optarg;
 				break;
 			default:
 				client_printf_err(sockfd, "unknown arguments\n");
-				ods_log_error("[%s] unknown arguments for %s command",
-						module_str, rollover_list_funcblock.cmdname);
+				ods_log_error("[%s] unknown arguments for rollover list command", module_str);
 				return -1;
 		}
 	}
@@ -213,5 +197,5 @@ run(int sockfd, cmdhandler_ctx_type* context, const char *cmd)
 }
 
 struct cmd_func_block rollover_list_funcblock = {
-	"rollover list", &usage, &help, NULL, &run
+	"rollover list", &usage, &help, NULL, NULL, &run, NULL
 };
