@@ -42,6 +42,8 @@
 #include "daemon/cfg.h"
 #include "libhsmdns.h"
 #include "db/key_data.h"
+#include "utilities.h"
+
 extern hsm_repository_t* parse_conf_repositories(const char* cfgfile);
 
 int verbosity;
@@ -53,15 +55,6 @@ usage(void)
     fprintf(stderr, "%s [-h] [-v] [-c <alternate-configuration>]\n", argv0);
 }
 
-typedef void (*functioncast_t)(void);
-extern functioncast_t functioncast(void*generic);
-
-functioncast_t
-functioncast(void*generic) {
-    functioncast_t* function = (functioncast_t*)&generic;
-    return *function;
-}
-
 /****************************************************************************/
 
 struct dblayer_struct {
