@@ -1,9 +1,36 @@
+/*
+ * Copyright (c) 2017 NLNet Labs. All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
+ * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
+ * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
+ * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
+ * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+ * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
+ * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ */
+
 #include "daemon/engine.h"
 #include "cmdhandler.h"
 #include "daemon/enforcercommands.h"
 #include "log.h"
 #include "str.h"
 #include "clientpipe.h"
+#include "longgetopt.h"
 #include "enforcer/enforce_task.h"
 #include "db/policy.h"
 
@@ -28,15 +55,11 @@ help(int sockfd)
 	);
 }
 
-/**
- * Purge
- * @param dbconn, Active database connection
- *
- * @return: error status, >0 on error
- */
 static int
-purge_policies(int sockfd, db_connection_t *dbconn)
+run(cmdhandler_ctx_type* context, int argc, char* argv[])
 {
+    int sockfd = context->sockfd;
+    db_connection_t* dbconn = getconnectioncontext(context);
 	policy_list_t* policy_list;
 	policy_t* policy;
 	zone_list_db_t* zonelist;
@@ -64,7 +87,7 @@ purge_policies(int sockfd, db_connection_t *dbconn)
 			client_printf(sockfd, "No zones on policy %s; purging...\n", name);
 			if (policy_delete(policy)) {
 				ods_log_crit("[%s] Error while purging policy from database", module_str);
-				client_printf(sockfd, "Error while updating database\n", name);
+				client_printf(sockfd, "Error while updating database\n");
 				result++;
 			}
 		}
@@ -74,17 +97,6 @@ purge_policies(int sockfd, db_connection_t *dbconn)
 	return result;
 }
 
-static int
-run(int sockfd, cmdhandler_ctx_type* context, const char *cmd)
-{
-    db_connection_t* dbconn = getconnectioncontext(context);;
-    engine_type* engine = getglobalcontext(context);
-    (void) cmd;
-
-    ods_log_debug("[%s] %s command", module_str, policy_purge_funcblock.cmdname);
-	return purge_policies(sockfd, dbconn);
-}
-
 struct cmd_func_block policy_purge_funcblock = {
-	"policy purge", &usage, &help, NULL, &run
+	"policy purge", &usage, &help, NULL, NULL, &run, NULL
 };
